package helper;

import java.text.DecimalFormat;

/**
 * This class encapsulates a piggy bank that can hold quarters and nickels (no
 * other coins). It provides a limited variety of services, such as adding to
 * and modifying the piggy bank.
 * 
 */
public class PiggyBank {

	private int quarters;
	private int nickels;

	private final int QUARTER_VALUE = 25;
	public int getValueOfAQuarter() {
		return QUARTER_VALUE;
	}

	private final int NICKEL_VALUE = 5;

	/**
	 * A flag that determines whether the return of toProperString is the number
	 * of nickels and the number nickels or the monetary value of the piggy
	 * bank. The default value is true.
	 **/
	private boolean asValue = true;

	/**
	 * Constructs a default piggy bank with the value of quarters as 0 and the
	 * number of nickels as 0.
	 **/
	public PiggyBank() {
		super();
		this.quarters = 0;
		this.nickels = 0;
	}

	/** Add one nickel to the piggy bank **/
	public void addANickel() {
		this.nickels = this.nickels + 1;
	}

	/** Add one quarter to the piggy bank **/
	public void addAQuarter() {
		this.quarters = this.quarters + 1;
	}

	/** Returns the number of quarters in the piggy bank **/
	public int getQuarters() {
		return quarters;
	}

	/**
	 * Sets the number of quarters in this piggybank to be the passed quantity.
	 * 
	 * @param quarters
	 *            the number of quarters that should be in this piggybank
	 * 
	 * @throws Exception
	 *             if the number of quarters is less than zerio
	 */
	public void setQuarters(int quarters) {
		crashQuarters(quarters < 0);
		this.quarters = quarters;
	}

	/** Returns the number of nickels in the piggy bank **/
	public int getNickels() {
		return nickels;
	}

	/**
	 * Sets the number of nickels in this piggybank to be the passed quantity.
	 * 
	 * @param nickels
	 *            the number of nickels that should be in this piggybank
	 * 
	 * @throws Exception
	 *             if the number of nickels is less than zerio
	 */
	public void setNickels(int nickels) {
		crashNickels(nickels < 0);
		this.nickels = nickels;
	}

	/** Returns the value of the nickels in the piggybank in cents **/
	public int getValueOfNickels() {
		return nickels * NICKEL_VALUE;
	}

	/** Returns the value of the quarters in the piggybank in cents **/
	public int getValueOfQuarters() {
		return quarters * QUARTER_VALUE;
	}

	public int getTotalValueOfPiggyBank() {
		return getValueOfNickels() + getValueOfQuarters();
	}

	/** Checks whether this piggy bank is equal to the passed piggy bank **/
	@Override
	public boolean equals(Object otherPiggyBank) {
		if (this == otherPiggyBank)
			return true;
		if (otherPiggyBank == null)
			return false;
		if (getClass() != otherPiggyBank.getClass())
			return false;
		PiggyBank other = (PiggyBank) otherPiggyBank;
		if (nickels != other.nickels)
			return false;
		if (quarters != other.quarters)
			return false;
		return true;
	}

	/**
	 * Checks whether the value of the piggy bank is equal to the passed piggy
	 * bank
	 **/
	public boolean hasSameValue(PiggyBank otherPiggyBank) {

		return this.getTotalValueOfPiggyBank() == otherPiggyBank
				.getTotalValueOfPiggyBank();
	}

	/**
	 * Sets whether this object, when printed out using its toProperString()
	 * method, shows itself as a monetary value or as a formatted string that
	 * lists the contents of this piggybank.
	 **/
	public void shouldShowValue(boolean shouldShowValue) {
		this.asValue = shouldShowValue;
	}

	/**
	 * Returns a string representation of this Piggy Bank that is either its
	 * monetary value or a formatted description of the contents of this
	 * piggybank (in terms of the number of quarters and nickels contained in
	 * this piggy bank). Which of these two options is used depends on this
	 * objects state with respect to shouldShowValue (see the method
	 * shoudShowValue(boolean). The default option is the first one.
	 **/
	public String toProperString() {
		DecimalFormat df = new DecimalFormat("0.00");
		return asValue ? "$"
				+ df.format((double) getTotalValueOfPiggyBank() / 100)
				: getNickels() + " nickels , and " + getQuarters()
						+ " quarters";
	}

	private void crashQuarters(boolean negQuarters) {
		if (negQuarters)
			try {
				throw new Exception("quarters cannot be negative");
			} catch (Exception e) {
				e.printStackTrace();

			}

	}

	private void crashNickels(boolean negNickels) {
		if (negNickels)
			try {
				throw new Exception("nickels cannot be negative");
			} catch (Exception e) {
				e.printStackTrace();
			}
	}

}
